package rabbit.awt;

// $Id: ImageComponent.java,v 1.3 2000/02/02 21:38:10 robo Exp $

import java.awt.*;
import java.io.*;

/** This class is a Component that holds an image.
 */
public class ImageComponent extends Component {
    /** @serial */
    private Image image;
   
    /** Create a new ImageComponent with the Image loaded from the
     *	given filename. 
     * @param filename the name of the image file.
     */
    public ImageComponent (String filename) {
	image = Toolkit.getDefaultToolkit ().getImage (filename);
	//prepareImage (image, this);
	MediaTracker mt = new MediaTracker (this);
	mt.addImage (image, 0);

	try {
	    mt.waitForAll ();
	} catch (InterruptedException e) {
	}	
    }

    /** Create a new ImageComponent with the Image loaded from the
     *	given bytearray. 
     * @param img the bytearray of the image.
     */
    public ImageComponent (byte[] img) {
	this (img, 0, img.length);
    }

    /** Create a new ImageComponent with the Image loaded from the
     *	given bytearray. 
     * @param img the bytearray of the image.
     * @param off the starting offset of the image.
     * @param len the length of the image.
     */
    public ImageComponent (byte[] img, int off, int len) {
	image = Toolkit.getDefaultToolkit ().createImage (img, off, len);
	MediaTracker mt = new MediaTracker (this);
	mt.addImage (image, 0);	
	try {
	    mt.waitForAll ();
	} catch (InterruptedException e) {
	}	
    }
    
    /** Get the preferred size of this component, that is the size of
     *	the image.
     * @return the size of the image.
     */
    public Dimension getPreferredSize () {
	if (image != null)
	    return new Dimension (image.getWidth (this), image.getHeight (this));
	return new Dimension (-1, -1);
    }
    
    /** Update the Graphics for this Component.
     * @param g the Graphics for this Component.
     */
    public void update (Graphics g) {
	paint (g);
    }
    
    /** Draw the image on the Graphics.
     * @param g the Graphics to draw the image on.
     */
    public void paint (Graphics g) {
	if (image != null)
	    g.drawImage (image, 0, 0, getSize ().width, getSize ().height, this);
    }

}
