package rabbit.filter;

// $Id: AccessFilter.java,v 1.2 2002/05/26 18:58:38 robo Exp $

import java.io.*;
import java.net.*;
import java.util.*;

import rabbit.proxy.*;
import rabbit.util.*;

/** This is a class that filters access based on ip adress.
 */
public class AccessFilter
{
    private static String accessfile;
    private static Vector iplist = new Vector ();
    private static final String DEFAULTCONFIG = "conf/access";

    /** Filter based on a socket.
     * @param s the Socket to check.
     * @return true if the Socket should be allowed, false otherwise.
     */
    public static boolean doIPFiltering (Socket s) {
	int isize =  iplist.size ();
	for (int i = 0; i < isize; i++) {
	    IPAccess ipa = (IPAccess)iplist.elementAt (i);
	    if (ipa.inrange (s.getInetAddress ()))
		return true;
	}
	return false;
    }

    /** Setup this class.
     * @param properties the Properties to get the settings from.
     */
    public static void setup (Properties properties) {
	loadAccess (properties.getProperty ("accessfile", DEFAULTCONFIG));	
    }
    
    /** Stop creation of this class
     */
    private AccessFilter () {
    }
    
    /** Read the data (accesslists) from a file.
     * @param filename the name of the file to read from.
     */
    private static void loadAccess (String filename) {
	filename = filename.replace ('/', File.separatorChar);
	accessfile = filename;
	
	try {
	    FileReader fr = new FileReader (accessfile);
	    loadAccess (fr);
	    fr.close ();
	} catch (IOException e) {
	    Proxy.logError (Proxy.FATAL, "Accessfile '" + accessfile + "' not found:(allowing no one is a bad idea) " + e);
	    System.exit (-1);
	}
    }
    
    /** Loads in the accessess allowed from the given Reader
     * @param r the Reader were data is available
     */
    public static void loadAccess (Reader r) throws IOException {
	Vector iplist = new Vector ();
	LineNumberReader br = new LineNumberReader (r);
	String line = null;
	while ((line = br.readLine ()) != null) {
	    // remove comments....
	    int index = line.indexOf ('#');
	    if (index >= 0) 
		line = line.substring (0,index);
	    if (line.equals (""))
		continue;
	    StringTokenizer st = new StringTokenizer (line);
	    if (st.countTokens () != 2) {
		Proxy.logError (Proxy.WARN, "Bad line in accessconf:" + br.getLineNumber());
		continue;
	    }
	    String low = st.nextToken ();
	    InetAddress lowip = null;
	    try {
		lowip = InetAddress.getByName (low);	    
	    } catch (UnknownHostException e) {
		Proxy.logError (Proxy.WARN, "Bad host: " + low + " at line:" + br.getLineNumber());
		continue;	    
	    }
		
	    String high = st.nextToken ();
	    InetAddress highip = null;
	    try {
		highip = InetAddress.getByName (high);
	    } catch (UnknownHostException e) {
		Proxy.logError (Proxy.WARN, "Bad host: " + high + " at line:" + br.getLineNumber());
		continue;	    
	    }
	    
	    if (lowip != null && highip != null) 
		iplist.addElement (new IPAccess (lowip, highip));
	}
	AccessFilter.iplist = iplist;
    }

    /** Saves the accesslist from the given Reader.
     * @param r the Reader with the users.
     */
    public static void saveAccess (Reader r) throws IOException {
	if (accessfile == null) 
	    return;
	BufferedReader br = new BufferedReader (r);
	PrintWriter fw = new PrintWriter (new FileWriter (accessfile));
	String line;
	while ((line = br.readLine ()) != null) 
	    fw.println (line);
	fw.flush ();
	fw.close ();
    }

    /** Get the list of ips allowed
     */
    public static Vector getIPList () {
	return iplist;
    }
}
