package rabbit.filter;

// $Id: AdFilter.java,v 1.6 2002/05/26 18:58:38 robo Exp $

import java.io.*;
import java.net.*;
import java.util.*;

import rabbit.html.*;
import rabbit.http.*;
import rabbit.proxy.*;
import rabbit.util.*;

/** This class switches advertising images into another image.
 */
public class AdFilter extends HTMLFilter {
    /** the image we replace ads with */
    protected static String ADREPLACER = "http://www.student.nada.kth.se/~d94-rol/pics/NoAd.gif";

    /** the actual imagelink. */
    private String adreplacer = null;

    /** Create a new AdFilter for the given request, response pair.
     * @param request the actual request made.
     * @param response the actual response being sent.
     */
    public AdFilter (HTTPHeader request, HTTPHeader response) {	
	super (request, response);
	int idx = -1;	
	adreplacer = Proxy.config.getProperty (getClass ().getName (), 
					       "adreplacer", 
					       ADREPLACER);
	while ((idx = adreplacer.indexOf ("$proxy")) > -1) {
	    adreplacer = adreplacer.substring(0,idx) + 
		Proxy.getHost ().getHostName () + ":" + Proxy.getPort () +
		adreplacer.substring (idx + "$proxy".length ());
	}
    }
    
    /** Removes advertising from the given block.
     * @param block the part of the html page we are filtering.
     */
    public void filterHTML (HTMLBlock block) {
	int astart = -1;
	
	Vector tokens = block.getTokens ();
	int tsize = tokens.size ();
	for (int i = 0; i < tsize; i++) {
	    Token t = (Token)tokens.elementAt (i);
	    switch (t.getType ()) {
	    case Token.TAG:
		Tag tag = t.getTag ();
		TagType tagtype = tag.getTagType ();
		if (tagtype == TagType.A) {
		    astart = i;
		    Tag atag = tag;
		    int ttsize = tokens.size ();
		    for (; i < ttsize; i++) {
			Token tk2 = (Token)tokens.elementAt (i);
			if (tk2.getType () == Token.TAG) {
			    Tag tag2 = tk2.getTag ();
			    TagType t2tt = tag2.getTagType ();
			    if (t2tt != null && 
				t2tt == TagType.SA)
				break;
			    else if (t2tt != null &&
				     t2tt == TagType.IMG) {
				if (isEvil (atag.getAttribute ("HREF")))
				    tag2.setAttribute ("src", adreplacer);
			    }
			}
		    }
		    if (i == tsize && astart < i) {
			block.setRest (((Token)tokens.elementAt (astart)).getStartIndex ());
		    }
		} else if (tagtype == TagType.LAYER || tagtype == TagType.SCRIPT) {
		    String src = tag.getAttribute ("src");
		    if (isEvil (src))
			tag.setAttribute ("src", adreplacer);
		}
	    default:
	    }	    
	}
    }
    
    /** Check if a string is evil (that is its probably advertising).
     * @param str the String to check.
     */
    public boolean isEvil (String str) {
	if (str == null)
	    return false;
	str = str.toLowerCase ();
	StringTokenizer st = new StringTokenizer (
	    Proxy.config.getProperty (getClass ().getName (),
				      "adlinks",
				      ""), 
		",");
	while (st.hasMoreElements ()) {
	    if (str.indexOf (st.nextToken ()) >= 0) {
		return true;
	    }
	}
	return false;
    }
}

