package rabbit.filter;

// $Id: BodyFilter.java,v 1.5 2002/05/26 18:58:38 robo Exp $

import java.io.*;
import java.net.*;
import java.util.*;

import rabbit.html.*;
import rabbit.http.*;
import rabbit.proxy.*;
import rabbit.util.*;

/** A class that inserts some text and links at the top of a page.
 *  Useful for inserting links to unfiltered page.
 */
public class BodyFilter extends HTMLFilter {
    private boolean done = false;

    /** the identifier for pages filtered with this proxy. */
    private static final String PREFIX = "This page is filtered by RabbIT ";
    /** the string to append after our advertising. */
    private static final String POSTFIX = "<br>";
    /** the link string. */
    private static final String LINK = "unfiltered page";

    /** Create a new BodyFilter for the given request, response pair.
     * @param request the actual request made.
     * @param response the actual response being sent.
     */
    public BodyFilter (HTTPHeader request, HTTPHeader response) {	
	super (request, response);
    }
    
    /** Insert some text at the top of the html page.
     * @param block the part of the html page we are filtering.
     */
    public void filterHTML (HTMLBlock block) {
	if (!done) {
	    Vector tokens = block.getTokens ();
	    int tsize = tokens.size ();
	    for (int i = 0; i < tsize; i++) {
		Token t = (Token)tokens.elementAt (i);
		switch (t.getType ()) {
		case Token.TAG:
		    Tag tag = t.getTag ();
		    if (tag.getTagType () == TagType.BODY) {
			i = insertTokens (block, i+1);
			done = true;
		    }
		default:
		}
	    }
	}
    }
    
    /** Insert the links in an ordered fashion.
     * @param block the html block were filtering.
     * @param pos the position in the block were inserting stuff at.
     * @return the new position in the block.
     */
    protected int insertTokens (HTMLBlock block, int pos) {
	block.insertToken (new Token 
			   (Proxy.config.getProperty 
			    (getClass ().getName (),
			     "prefix", 
			     PREFIX)),
			   pos++);
	if (Proxy.config.getProperty (getClass ().getName (),
				      "unfilteredlink",
				      "true").toLowerCase ().equals ("true")) {
	    Tag a = new Tag ("A");
	    try {
		URL url = new URL (request.getRequestURI ());
		a.addArg ("HREF",
			  "\"" + url.getProtocol() + "://noproxy." + url.getHost () + 
			  ((url.getPort () > 0) ? ":" + url.getPort () : "") +
			  url.getFile () + 
			  ((url.getRef() != null) ? "#" + url.getRef () : "") + "\"");
		block.insertToken (new Token (a), pos++);
		block.insertToken (new Token 
				   (Proxy.config.getProperty 
				    (getClass ().getName (),
				     "link",
				     LINK)),
				   pos++);
		Tag slasha = new Tag ("/A");
		block.insertToken (new Token (slasha), pos++);
	    } catch (MalformedURLException e) {
		// ignore
	    }
	}
	block.insertToken (new Token 
			   (Proxy.config.getProperty 
			    (getClass ().getName (),
			     "postfix",
			     POSTFIX)),
			   pos++);
	return pos;
    }
}
