package rabbit.filter;

// $Id: DontFilterFilter.java,v 1.2 2001/07/08 17:35:16 robo Exp $

import java.io.*;
import java.net.*;
import java.util.*;

import rabbit.http.*;
import rabbit.proxy.*;
import rabbit.util.*;

/** This is a class that makes sure the proxy doesnt filter certain pages.
 *  It matches pages based on the URL.
 *  <p>
 *  It uses the config option <tt>dontFilterURLmatching</tt> with a default value
 *  of the empty string. This value is considered a comma ',' separated list.<br>
 *  Matching is done with indexOf so no regular expressions are used for now.
 */
public class DontFilterFilter 
{
    private static Properties config = new Properties ();
   
    /** To block creation of this class.
     */
    private DontFilterFilter () {
    }
    
    /** Test if a socket/header combination is valid or return a new HTTPHeader.
     *  If the request matches a certain criteria dont filter it. This filter is 
     *  Good for pages with broken HTML that would wreck the HTML parser. 
     * @param socket the Socket that made the request.
     * @param header the actual request made.
     * @param con the Connection handling the request.
     * @return null
     */
    public static HTTPHeader doHTTPInFiltering (Socket socket, HTTPHeader header, Connection con) {
	StringTokenizer st = new StringTokenizer (config.getProperty ("dontFilterURLmatching", ""), ",");
	while (st.hasMoreTokens ()) {
	    if (header.getRequestURI ().indexOf (st.nextToken ()) > -1)
		con.setMayFilter (false);
	}	
	return null;
    }
    
    /** test if a socket/header combination is valid or return a new HTTPHeader.
     * @param socket the Socket that made the request.
     * @param header the actual request made.
     * @param con the Connection handling the request.
     * @return null if everything is fine or a HTTPHeader describing the error (like a 403).
     */
    public static HTTPHeader doHTTPOutFiltering (Socket socket, HTTPHeader header, Connection con) {
	return null;
    }

    /** Setup this class with the given properties.
     * @param properties the new configuration of this class.
     */
    public static void setup (Properties properties) {
	config = properties;
    }
}
