package rabbit.filter;

// $Id: ProxyAuth.java,v 1.2 2002/07/08 20:09:52 robo Exp $

import java.io.*;
import java.net.*;
import java.util.*;

import rabbit.http.*;
import rabbit.proxy.*;
import rabbit.util.*;

/** This is a filter that requires users to use proxy-authentication.
 */
public class ProxyAuth 
{
    private static Hashtable users;

    /** test if a socket/header combination is valid or return a new HTTPHeader.
     *  Check that the user has been authenticate..
     * @param socket the Socket that made the request.
     * @param header the actual request made.
     * @param con the Connection handling the request.
     * @return null if everything is fine or a HTTPHeader describing the error (like a 403).
     */
    public static HTTPHeader doHTTPInFiltering (Socket socket, HTTPHeader header, Connection con) {
	if (con.getMeta ())
	    return null;
	String username = con.getUserName ();
	String pwd = con.getPassWord ();
	if (username == null || pwd == null) {
	    try {
		return con.get407 ("internet", new URL (header.getRequestURI ()));
	    } catch (MalformedURLException e) {
		Proxy.logError (Proxy.WARN, "Bad url: " + e);
	    }
	}
	String rpwd = (String)users.get (username);
	if (rpwd == null || !rpwd.equals (pwd)) {
	    try {
		return con.get407 ("internet", new URL (header.getRequestURI ()));
	    } catch (MalformedURLException e) {
		Proxy.logError (Proxy.WARN, "Bad url: " + e);
	    }
	}
	
	return null;
    }

    /** test if a socket/header combination is valid or return a new HTTPHeader.
     *  does nothing.
     * @param socket the Socket that made the request.
     * @param header the actual request made.
     * @param con the Connection handling the request.
     * @return null if everything is fine or a HTTPHeader describing the error (like a 403).
     */
    public static HTTPHeader doHTTPOutFiltering (Socket socket, HTTPHeader header, Connection con) {
	return null;
    }    

    /** Setup this class with the given properties.
     * @param properties the new configuration of this class.
     */
    public static void setup (Properties properties) {
	String userfile = properties.getProperty ("userfile", "conf/allowed");
	try {
	    users = HTTPBaseFilter.loadUsers (new FileReader (userfile), false);
	} catch (FileNotFoundException e) {
	    Proxy.logError (Proxy.WARN, "could not load the users file: '" + userfile + "'. " + e);
	} catch (IOException e) {
	    Proxy.logError (Proxy.WARN, "Error while loading the users file: '" + userfile + "'. " + e);	    
	}	
    }
}
