package rabbit.handler;

// $Id: FilterHandler.java,v 1.9 2002/07/16 22:10:17 robo Exp $

import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import java.util.zip.*;

import rabbit.filter.*;
import rabbit.html.*;
import rabbit.http.*;
import rabbit.io.*;
import rabbit.proxy.*;
import rabbit.proxy.Proxy;

/** This class is used to filter html pages.
 */
public class FilterHandler extends GZIPHandler
{
    private static Properties config = new Properties ();
    private static Vector filterclasses = new Vector ();

    /** Create a new FilterHandler for the given request.
     * @param con the Connection handling the request.
     * @param request the actual request made.
     * @param response the actual response.
     * @param contentstream the stream to read data from.
     * @param clientstream the stream to write data to.
     * @param maycache May we cache this request? 
     * @param mayfilter May we filter this request?
     * @param size the size of the data beeing handled.
     */
    public FilterHandler (Connection con, 
			  HTTPHeader request, 
			  HTTPHeader response, 
			  InputStream contentstream, 
			  MultiOutputStream clientstream,
			  boolean maycache,
			  boolean mayfilter, 
			  long size) {
	super (con, request, response, contentstream, clientstream, maycache, mayfilter, size);
	
	response.removeHeader ("Content-Length");
	if (!con.getChunking ())
	    con.setKeepalive (false);
    }

    /** Send the actual data (read data, filter data, send data).
     * @throws IOException if reading or writing of the data fails.
     */ 
    public void send () throws IOException {
	byte[] v = new byte[10240]; // 10kB blocks should be good.
	int read;
	Vector filters = initFilters ();
	HTMLParser parser = new HTMLParser ();
	HTMLBlock block = null;
	long total = 0;
	int start = 0; 
	int len = v.length;

	while ((size < 0 || total < size) && (read = contentstream.read(v, start, len)) > 0) {
	    total += read;
	    read += start;
	    parser.setText (v, read);
	    try {
		block = parser.parse ();
		int fsize = filters.size ();
		for (int i = 0; i < fsize; i++) {
		    HTMLFilter hf = (HTMLFilter) filters.elementAt (i);
		    hf.filterHTML (block);
		}
		block.send (clientstream);
	    } catch (HTMLParseException e) {
		Proxy.logError (Proxy.INFO,"Bad HTML: " + e.toString ());
		clientstream.write (v, 0, read);
	    }
	    
	    // handle any trailing parts...
	    if (block != null && block.restSize () > 0) {
		start = block.restSize ();
		if (start == v.length) 		    
		    v = new byte[v.length + 10240]; // grow slowly if needed....
		len = v.length - start;	
		block.insertRest (v);		
	    } else {
		start = 0; 
		len = v.length;
	    }

	    if (size > -1) {
		long l = size - total;
		if (l < len)
		    len = (int)l;
	    }
	}
	if (block != null)
	    block.sendRest (clientstream);
	if (size > 0 && total != size)
	    setPartialContent (total, size);
    }

    /** Setup this class.
     * @param prop the properties of this class.
     */
    public static void setup (Properties prop) {
	config = prop;
	
	StringTokenizer st = new StringTokenizer (config.getProperty ("filters", ""), ",");
	filterclasses = new Vector ();
	String classname = "unknown";
	while (st.hasMoreTokens ()) {
	    try {
		classname = st.nextToken ();
		Class cls = Class.forName (classname);
		filterclasses.addElement (cls);
	    } catch (ClassNotFoundException e) {
		Proxy.logError (Proxy.WARN, "Could not find filter: '" + classname + "'");
	    }
	}
    }

    /** Initialize the filter we are using.
     * @return a Vector of HTMLFilters.
     */
    protected Vector initFilters () {
	Vector fl = new Vector ();
	Class cls = null;
	Class types[] = new Class[2];
	types[0] = request.getClass ();
	types[1] = response.getClass ();
	Object args[] = new Object[2];
	args[0] = request;
	args[1] = response;

	int fsize = filterclasses.size ();
	for (int i = 0; i < fsize; i++) {
	    try {
		cls = (Class)filterclasses.elementAt (i);
		Constructor cons = cls.getConstructor (types);
		fl.addElement (cons.newInstance (args));
	    } catch (NoSuchMethodException e) {
		Proxy.logError (Proxy.WARN, "Could not find constructor for: '" + cls + "' " + e);
	    } catch (IllegalAccessException e) {
		Proxy.logError (Proxy.WARN, "Could not get constructor for: '" + cls + "' " + e);
	    } catch (InvocationTargetException e) {
		Proxy.logError (Proxy.WARN, "Could not invoke constructor for: '" + cls + "' " + e);
	    } catch (InstantiationException e) {
		Proxy.logError (Proxy.WARN, "Could not instanciate class: '" + cls + "' " + e);		
	    }
	}
	return fl;
    }
}
