package rabbit.handler;

// $Id: GZIPHandler.java,v 1.4 2001/11/17 20:37:54 robo Exp $

import java.io.*;
import java.lang.reflect.*;
import java.util.*;
import java.util.zip.*;

import rabbit.filter.*;
import rabbit.html.*;
import rabbit.http.*;
import rabbit.io.*;
import rabbit.proxy.*;

/** This class is used to gzip streams.
 */
public class GZIPHandler extends BaseHandler
{
    private static boolean compress = true;
    private GZIPOutputStream gz = null;
    private boolean isCompressing = false;

    /** Create a new GZIPHansler for the given request.
     * @param con the Connection handling the request.
     * @param request the actual request made.
     * @param response the actual response.
     * @param contentstream the stream to read data from.
     * @param clientstream the stream to write data to.
     * @param maycache May we cache this request? 
     * @param mayfilter May we filter this request?
     * @param size the size of the data beeing handled.
     */
    public GZIPHandler (Connection con, 
			HTTPHeader request, 
			HTTPHeader response, 
			InputStream contentstream, 
			MultiOutputStream clientstream,
			boolean maycache,
			boolean mayfilter,
			long size) {
	super (con, request, response, contentstream, clientstream, maycache, mayfilter, size);
	
	if (compress) {
	    response.removeHeader ("Content-Length");
	    String gzip = response.getHeader ("Content-Encoding");
	    isCompressing = !(gzip != null 
			      && (gzip.equalsIgnoreCase ("gzip") 
				  || (gzip.equalsIgnoreCase ("compress"))));
	    if (isCompressing) {
		response.setHeader ("Content-Encoding", "gzip");
		if (!con.getChunking ())
		    con.setKeepalive (false);
	    }
	}
    }

    /** This method is used to prepare the stream for the data being sent.
     *  This method sets the stream up for gzipping.
     */
    protected void prepareStream () throws IOException {
	if (isCompressing) {
	    gz = new GZIPOutputStream (clientstream);
	    clientstream = new MultiOutputStream (gz);
	}
    }
    
    /** This method is used to finish the stream for the data being sent.
     *  This method finishes the gzipping.
     */
    protected void finishStream () throws IOException {
	if (isCompressing)
	    gz.finish ();
    }

    /** Setup this class.
     * @param prop the properties of this class.
     */
    public static void setup (Properties prop) {
	String comp = prop.getProperty ("compress", "true");
	if (comp.equalsIgnoreCase ("false"))
	    compress = false;
	else
	    compress = true;
    }
}




