package rabbit.io;

// $Id: MaxSizeInputStream.java,v 1.2 2000/01/08 20:29:39 robo Exp $

import java.io.*;

/** This class allows setting the size of the inputstream to a 
 *  maximum nuber of bytes.
 */
public class MaxSizeInputStream extends InputStream 
{
    private long maxSize = -1;
    private long current = 0;
    private InputStream is = null;

    /** Creates a new MaxSizeInputStream with a given limit on the 
     *  given stream.
     */
    public MaxSizeInputStream (InputStream is, long l) {
	current = 0; 
	maxSize = l;
	this.is = is;
    }

    /** Get the number of bytes ready for reading.
     * @return the number of bytes that can be read without blocking.
     * @throws IOException if an I/O error occurs.
     */
    public int available () throws IOException {
	int a = is.available ();
	if (a < (maxSize - current))
	    return a;
	return (int)(maxSize - current);
    }
    
    /** Reads the next byte of data from this input stream. The value
     *  byte is returned as an int in the range 0 to 255. If no byte is
     *  available because the end of the stream has been reached, the
     *  value -1 is returned. This method blocks until input data is
     *  available, the end of the stream is detected, or an exception
     *  is thrown.
     * @return the next byte of data, or -1 if the end of the stream
     *  is reached. 
     * @throws IOException if an I/O error occurs. 
     */
    public int read () throws IOException {
	if (current < maxSize) {
	    int ret = is.read ();
	    current++;
	    return ret;
	}
	return -1;
    }
    
    /** Read a block of data (up to b.length bytes).
     * @return the number of byte actually read.
     * @throws IOException if an I/O error occurs. 
     */
    public int read (byte[] b) throws IOException {
	return read (b, 0, b.length);
    }

    /** Read a block of data (up to len bytes).
     * @return the number of byte actually read.
     * @throws IOException if an I/O error occurs. 
     */
    public int read (byte[] b, int off, int len) throws IOException {
	if (len > (int)(maxSize - current)) {
	    len = (int)(maxSize - current);
	}
	int ret = is.read (b, off, len);
	current += len;
	return ret;
    }

    /** Skip a number of bytes.
     * @return the number of byte actually skipped.
     * @throws IOException if an I/O error occurs. 
     */
    public long skip (long n) throws IOException {
	if (n > (maxSize - current))
	    n = maxSize - current;
	long r = is.skip (n);
	current += r;
	return r;
    }
}
