package rabbit.io;

// $Id: MaxSizeOutputStream.java,v 1.2 2000/01/05 21:06:23 robo Exp $

import java.io.*;

/** A class that doesnt create outputstreams of more than a specified
 *  amount of data, the rest is simply swallowed
 */
public class MaxSizeOutputStream extends OutputStream 
{
    private long maxSize = -1;
    private long current = 0;
    private OutputStream os = null;

    /** Creates a new MaxSizeOutputStream with a given limit and
     *	connects it to a given stream.
     */
    public MaxSizeOutputStream (OutputStream os, long l) {
	current = 0;
	maxSize = l;
	this.os = os;
    }    

    /** Write a byte to this stream.
     * @param b the byte to write.
     * @throws IOException if the underlying stream does. 
     */
    public void write(int b) throws IOException {
	if (current < maxSize)
	    os.write (b);
	current++;
    }
    
    /** Write a byte array to this stream.
     * @param b the byte array to write.
     * @throws IOException if the underlying stream does. 
     */
    public void write(byte[] b) throws IOException {
	if ((current + b.length) < maxSize)
	    os.write (b);
	current += b.length;
    }
    
    /** Write a byte array to this stream.
     * @param b the byte to write.
     * @param off the starting offset.
     * @param len the number of bytes to write.
     * @throws IOException if the underlying stream does. 
     */
    public void write(byte[] b, int off, int len) throws IOException {
	if ((current + len) < maxSize) {
	    os.write (b, off, len);
	    current += len;
	}
	else if (maxSize - len < current) {
	    len = (int)(maxSize - current);
	    os.write (b, off, len);
	    current += len;
	}
    }
    
    /** Write any pending data.
     * @throws IOException if the underlying stream does. 
     */
    public void flush() throws IOException {
	os.flush ();
    }
    
    /** Close this stream.
     * @throws IOException if the underlying stream does. 
     */
    public void close() throws IOException {
	os.close ();
    }

    /** Simple self test
     */
    public static void main (String args[]) {
	try {
	    
	    FileOutputStream fis1 = new FileOutputStream ("hepp");
	    MaxSizeOutputStream msos1 = new MaxSizeOutputStream (fis1,1);
	    
	    FileOutputStream fis2 = new FileOutputStream ("hopp");
	    MaxSizeOutputStream msos2 = new MaxSizeOutputStream (fis2,2);

	    FileOutputStream fis3 = new FileOutputStream ("happ");
	    MaxSizeOutputStream msos3 = new MaxSizeOutputStream (fis3,4711);
	    
	    PrintWriter p1 = new PrintWriter (msos1);
	    PrintWriter p2 = new PrintWriter (msos2);
	    PrintWriter p3 = new PrintWriter (msos3);
	    p1.println("tjo och hej i lingonskogen");
	    p1.flush ();
	    p1.close ();
	    p2.println("tjo och hej i lingonskogen");
	    p2.flush ();
	    p2.close ();
	    p3.println("tjo och hej i lingonskogen");
	    p3.flush ();
	    p3.close ();
	} catch (Exception e) {
	    System.out.println ("An error occured: " + e);
	} 
    }
}
