package rabbit.meta;

// $Id: FileSender.java,v 1.2 2001/12/09 22:00:06 robo Exp $

import java.io.*;
import java.net.*;
import java.util.*;

import rabbit.html.*;
import rabbit.http.*;
import rabbit.io.*;
import rabbit.proxy.*;

/** A class to send files from the htdocs directory.
 *  This makes RabbIT act as a very simple web server.
 */
public class FileSender implements MetaHandler 
{
    /** handler of a MetaPage, that is a special page the proxy supports (like the status page).
     * @param input the InputStream from the client.
     * @param output the OutputStream to the client.
     * @param header the http request header
     * @param htab the supplied argument to the page (CGI-parameters).
     * @param con the Connection that is serving the request.
     */
    public void handle (InputStream input, 
			MultiOutputStream output, 
			HTTPHeader header, 
			Properties htab, 
			Connection con) {
	String file = htab.getProperty ("argstring");
	if (file == null) 
	    throw (new IllegalArgumentException ("no file given."));
	
	if (file.indexOf ("..") >= 0)    // file is un-url-escaped 
	    throw (new IllegalArgumentException ("Bad filename given"));

	String filename = "htdocs/" + file;
	if (filename.endsWith ("/"))
	    filename = filename + "index.html";
	filename = filename.replace ('/', File.separatorChar);
	HTTPHeader rheader = con.getHeader ();
	if (filename.endsWith ("gif"))
	    rheader.setHeader ("Content-type", "image/gif");
	else if (filename.endsWith ("jpeg") || filename.endsWith ("jpg"))
	    rheader.setHeader ("Content-type", "image/jpeg");
	else if (filename.endsWith ("txt"))
	    rheader.setHeader ("Content-type", "text/plain");
	File fle = new File (filename);
	rheader.setHeader ("Content-Length", "" + fle.length ());
	con.setContentLength (rheader.getHeader ("Content-Length"));

	InputStream fis;
	try {
	    fis = new FileInputStream (filename);
	} catch (IOException e) {
	    throw (new IllegalArgumentException ("could not open file: " + file + "."));	    
	}
	try {
	    output.writeHTTPHeader (rheader);
	    byte buffer[] = new byte[10240];
	    int i;
	    while ((i = fis.read (buffer)) >= 0) {
		output.write (buffer, 0, i);
	    }
	} catch (IOException e) {
	    throw (new IllegalArgumentException ("could not send: " + file + "."));
	}
    }
}
