package rabbit.util;

// $Id: RestartableThread.java,v 1.3 2002/01/04 23:05:18 robo Exp $

/** A thread class that is restartable.
 */
public abstract class RestartableThread extends Thread {
    private boolean started = false;
    private Object lock = new Object ();

    public RestartableThread () {
    }

    public RestartableThread (String name) {
	super (name);
    }
    
    /** This is the method that is run. 
     *  To change a thread to a restartablethread move the 
     *  old run to doWork.
     */
    public abstract void doWork ();

    /** Dont override this in subclasses. 
     */
    public void run () {
	while (true) {
	    doWork ();
	    try {
		synchronized (lock) {	    
		    lock.wait ();
		}
	    } catch (InterruptedException e) {
		// ignore;
	    }
	}
    }

    /** Overridden to handle restarting of this thread.
     */ 
    public void start () {
	if (started) {
	    synchronized (lock) {
		lock.notify ();
	    }
	} else {
	    started = true;
	    super.start ();
	}
    }
}
